/* Defines for cool interrupt-driven serial routines */

#define BYTE  unsigned char
#define WORD  unsigned short
#define DWORD unsigned long

typedef int X10_UNIT;
typedef int X10_UNIT_BITMAP;

#define EXTC_MAX 3

#ifdef DOS
#define DEVICE_DEFAULT "COM1"
#else
#define DEVICE_DEFAULT "/dev/cm11"
#endif

#define FN_ALL_UNITS_OFF   0x00
#define FN_ALL_LIGHTS_ON   0x01
#define FN_ON              0x02
#define FN_OFF             0x03
#define FN_DIM             0x04
#define FN_BRIGHT          0x05
#define FN_ALL_LIGHTS_OFF  0x06
#define FN_EXT_CODE        0x07
#define FN_HAIL_REQUEST    0x08
#define FN_HAIL_ACK        0x09
#define FN_PRESET_DIM_0    0x0a
#define FN_PRESET_DIM_1    0x0b
#define FN_EXT_DATA_XFER   0x0c
#define FN_STATUS_ON       0x0d
#define FN_STATUS_OFF      0x0e
#define FN_STATUS_REQUEST  0x0f

#define EFN_SET_BRIGHTNESS 0x31
#define EFN_REQUEST_STATUS 0x37
#define EFN_STATUS         0x38

#define TIMEOUT_LIMIT 10

#define DEBUG_CM11          optDebug & 0x0001
#define DEBUG_SERIAL        optDebug & 0x0002
#define DEBUG_STREAM        optDebug & 0x0004
#define DEBUG_ALIAS         optDebug & 0x0008
#define DEBUG_MACRO         optDebug & 0x0010
#define DEBUG_SUNRISE       optDebug & 0x0020

struct tagEvent
{
    struct tagEvent *next;
    
    X10_UNIT        function;
    X10_UNIT_BITMAP deviceMask;
    int             dimLevel;
    int             dimAbsolute;
    BYTE            extended[EXTC_MAX];
};


/****** In stream.c ******/
// File I/O optimized for reading a stream of strings/words = "tokens"

typedef struct tagStream * STREAM;

STREAM OpenStream(char * filename);
STREAM StringStream(char *line);
void CloseStream(STREAM);
int GetNextToken(STREAM, char *, int size);
void StreamError(STREAM);
int IsEndOfStream(STREAM);

/************** In front.c *****************/

extern int optDebug;
extern int optVerbose;
extern int optMonitoredHouseCode;
extern int optDownloadClock;
extern int optTrackStatus;

/****** In cm11.c ******/

void Cm11ServiceLoop(int returnOnQuiet);    // Go to cm11 service loop
void Cm11InsertSimpleEvent(BYTE header, BYTE code);
void InsertExtendedEvent(BYTE header, BYTE code, BYTE *ext);
void EpromDownload(BYTE *eprom, int epromLength);
int ImmediateCommand(char *line);
int ImmediateStream(STREAM);

/****** In serial.c ******/

unsigned char GetSerialCharWait();
int GetSerialChar();
int PutSerialChar(BYTE c);
int DataAvailable();
int DataAvailableWait(int timeoutSeconds);
int DataQueued();  /* # chars waiting to go out */
int UninstallSerialDrivers(void);
int InstallSerialDrivers(char * path);
int WriteSerial(unsigned char *c,int len);
int DataPurge();

/****** In alias.c ******/

char *SpaceDrop(char * str);         // Generic routines
char *NotSpaceDrop(char *str);
int Hex2Bytes(char * in, BYTE * out, int size);

// Low-level conversion to X10 units
extern char gHouseChr[];
extern int deviceInt[];
X10_UNIT ChrToHouseCode(char chr); // Single character -> housecode (hi-nyb)
X10_UNIT IntToDeviceCode(int device); // The device (number part, low-nybble)

// Higher-level conversion, including aliasing
char *UnitToStr(X10_UNIT unit, char *str, int len); // single unit -> string
char *UnitsToStr(X10_UNIT house, X10_UNIT_BITMAP codeMap, char *str, int len);
int StrToUnits(char *in, X10_UNIT *house, X10_UNIT_BITMAP *devices);
X10_UNIT StrToUnit(char *in);

// Add a single alias to table
int AddAlias(char *alias, char *translation);
// Clean up!
void CleanupAliasStuff();
int DumpAliases(FILE *);


/****** In macro.c ******/

int ReadMacroFile(STREAM inStream);  // "Compile" a macro/timer file
                                     // returns 0 if error

int CreateEpromImage(BYTE **eprom);  // "Link" and create EPROM image
                                     // returns size, -1 if error

void DumpSymbols(BYTE *s , int size, char *symFile);  // Dump symbols to file
void LookupSymbol(unsigned long address, char *buffer, int size);

void CleanupMacroStuff();            // Release resources allocated by x10mac

// Interpret an Event (used for immediate commands)
struct tagEvent *ReadEvent(STREAM inStream);

// Extended module bitmap (256-bit)
extern X10_UNIT_BITMAP gExtModules[];

/****** In sunrise.c ******/

struct tagSunriseBin {
    int sunriseMinutes;
    int sunsetMinutes;
    int lastDay;
};

int SunsetStrip(struct tagSunriseBin *bins, int maxBins, int toleranceMinutes);




