#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "version.h"

/* TODO:
   Open serial port blocking
   Socket operation
   Integrate sunrise (in macro code)
 */

#include "cm11.h"

//static char revision[]="$Revision: 1.2 $"; /* RCS Info */

#define BUFFER_SIZE 64
int optDebug = 0;
int optVerbose = 0;
X10_UNIT optMonitoredHouseCode = 0x60;  // "A"
int optDownloadClock = 0;
int optTrackStatus = 0;

int main(int argc,char **argv)
{
    STREAM inStream;
    char *argv0 = *argv++;
    
    int pipe=0 ;
    int optMonitor = 0;
    int optNoTalk = 0;
    int optDoEprom = 0;
    int optImmediate = 0;
    char symbolFile[256];

    strcpy(symbolFile, getenv("HOME"));
    strcat(symbolFile, "/.cm11-symbols");
    
    if (argc-- == 1) {
        /* no entry: give usage */
        fprintf(stdout,"Usage: %s [-] [-v|q] [cm11 sources]\n"
                       "       %s [-] [-v|q] -i [immediate commands]\n",
                argv0, argv0);
        return 0;
    }
    while ((argc > 0) && *argv[0] == '-') {
        int flushLine = 0;
        char *s = (*argv++) + 1;
        --argc;
        
        if (*s == '\0') {
            if (optVerbose)
                fprintf(stderr,"Piping from stdin.\n");
            pipe = 1;
        }
        
        while (!flushLine && *s !='\0') {
                
            switch (*s++) {
            case 'h' :
                fprintf(stdout,"%s:  X10 CM11a Assembler & Interface.\n"
                        "       (c) 1997 Erik D. Olson\n",argv0);
                fprintf(stdout,"Options:\n"
                        "  -h        Print this help\n"
                        "  -v|q      Verbose|quiet mode (default %s)\n"
                        "  -s        Sync interface clock to system clock\n"
                        "  -S        Show interface clock\n"
                        "  -i xcmd   Execute single immediate command\n"
                        "  -y file   Alt symbol file (default = %s)\n"
			"  -V        Print version number and exit\n"
                        ,
                        optVerbose?"verbose":"quiet",
                        symbolFile);
            
                break;
	    case 'V':
		printf("%s\n",cm11_version);
		exit(0);
            case 's' :
                if (optVerbose) {
                    fprintf(stderr,"Sync clocks.  Monitored housecode=%c\n",
                            gHouseChr[optMonitoredHouseCode >> 4]);
                }
                optDownloadClock = 1;
                break;
          
            case 'S' :
                optTrackStatus = 1;
                if (optVerbose) {
                    fprintf(stderr,"Verify status/clock\n");
                }
                break;
                
            case 'e' :
                optDoEprom = 1;
                if (optVerbose) {
                    fprintf(stderr, "Create Eprom Image\n");
                }
                
                break;
                
            case 'v' :
                optVerbose=1;
                fprintf(stderr,"Verbose mode.\n");
                break;
            case 'q' :
                optVerbose=0;
                break;
            case 'm' :
                optMonitor = 1;
                if (optVerbose) {
                    fprintf(stderr, "Monitor on.\n");
                }
                break;
            case 'H' :
                flushLine = 1;
                if (*s=='\0') {
                    if (argc--) s=*argv++; else break;
                }
                if (optVerbose) {
                    fprintf(stderr,"Monitored housecode = %c\n", *s);
                }
                optMonitoredHouseCode = ChrToHouseCode(*s);
                break;
                
            case 'n' :
                optNoTalk = 1;
                if (optVerbose) {
                    fprintf(stderr, "No talking to interface\n");
                }
                break;
                
            case 'i' :
                optImmediate = 1;
                if (optVerbose)
                    fprintf(stderr,"Immediate commands.\n");
                
                break;

            case 'a' :
                flushLine = 1;
                if (*s=='\0') {
                    if (argc--) s=*argv++; else break;
                }
                if (optVerbose)
                    fprintf(stderr,"Assembling %s\n",s);
                inStream = OpenStream(s);
                
                if (inStream) {
                    ReadMacroFile(inStream);
                    CloseStream(inStream);
                }
                break;

            case 'y':
                flushLine = 1;
                if (*s=='\0') {
                    if (argc--) s=*argv++; else break;
                }
                
                strncpy(symbolFile, s, sizeof(symbolFile));
                
                if (optVerbose) {
                    fprintf(stderr, "Symbols to %s\n", symbolFile);
                }
                break;
                    
            case 'd' :
                flushLine = 1;
                if (*s=='\0') {
                    if (argc--) s=*argv++; else break;
                }
                
                optDebug = atoi(s);
                if (optVerbose) {
                    fprintf(stderr, "Debug set to %d\n", optDebug);
                }
                break;
                    
            default:
                fprintf(stderr,"Illegal option.  Try %s -h for help.\n",argv0);
                break;
            } /* switch */
        } /* args with - */
    }
    
    // Get symbols if interpreting immediate commands
    if (optImmediate) {
        inStream = OpenStream(symbolFile);
        if (inStream) {
            ReadMacroFile(inStream);
            CloseStream(inStream);
        }
    }
    
    if (pipe) {
        if (optImmediate) {
            if (optVerbose)
                fprintf(stderr,"Immediate stdin\n");
            inStream = OpenStream(NULL);

            if (inStream) {
                ImmediateStream(inStream);
                CloseStream(inStream);
            }
        } else {
            
            if (optVerbose)
                fprintf(stderr,"Assemble stdin\n");
            inStream = OpenStream(NULL);
            if (inStream) {
                ReadMacroFile(inStream);
                CloseStream(inStream);
            }
        }
    } else while (argc--) {
        if (optImmediate) {
            if (optVerbose)
                fprintf(stderr,"Immediate Command %s\n",*argv);
            ImmediateCommand(*argv++);
        } else {
            if (optVerbose)
                fprintf(stderr,"Assemble file %s\n",*argv);
            inStream = OpenStream(*argv++);
            if (inStream) {
                ReadMacroFile(inStream);
                CloseStream(inStream);
            }
        }
        
    } 
    if (!optNoTalk) {
	char *device = getenv("CM11DEV");
	if (!device)
	    device = DEVICE_DEFAULT;
        if (InstallSerialDrivers(device)) {
            return 1;
        }
    }

    if (optDoEprom) {
        BYTE *eprom;
        int epromLength;
        epromLength = CreateEpromImage(&eprom);
        if (optVerbose) {
            fprintf(stderr,"Eprom length = %x\n", epromLength);
            
        }
        DumpSymbols(eprom, epromLength, symbolFile);
        
        if (epromLength > 0 && !optNoTalk) {
            EpromDownload(eprom, epromLength);
        }
    } else if (!optImmediate) {
        // OK, we're not creating the eprom image.  Maybe doing MONITOR
        // mode or maybe just downloading the clock.  In either case,
        // we need those symbols now!

        inStream = OpenStream(symbolFile);
        if (inStream) {
            ReadMacroFile(inStream);
            CloseStream(inStream);
        }
    }
    
    
    if (!optNoTalk) {
        Cm11ServiceLoop(!optMonitor);
        UninstallSerialDrivers();
    }
    CleanupMacroStuff();
    CleanupAliasStuff();

    return 0;
} /* main */


