#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cm11.h"

//#define DEBUG 1
#define STREAM_BUFFER_SIZE 512

struct tagStream
{
    FILE *inF;
    char buffer[STREAM_BUFFER_SIZE];
    char name[64];
    int line;
    char *s;
};

struct tagStream * OpenStream(char * filename)
{
    STREAM s = calloc(1, sizeof(struct tagStream));

    if (!s) return NULL;

    if (filename == NULL) {
        s->inF = stdin;
	filename = "stdin";	/* for strncpy later */
    } else {
        s->inF = fopen(filename, "r");
    }
    
    if (s->inF == NULL) {
        free(s);
        return NULL;
    }
    strncpy(s->name, filename, sizeof(s->name)-1);
    s->line = 0;
    
    s->s = s->buffer;
    return s;
}

struct tagStream * StringStream(char *line)
{
    STREAM s = calloc(1, sizeof(struct tagStream));

    if (!s) return NULL;

    strncpy(s->buffer, line, sizeof(s->buffer));
    s->s = s->buffer;
    s->inF = NULL;
    s->line = 0;
    
    strncpy(s->name, "(string)", sizeof(s->name));
    return s;
    
}


void CloseStream(STREAM s)
{
    if (s->inF != stdin && s->inF != NULL) {
        fclose(s->inF);
    }
    free(s);
}

static void RefreshStreamBuffer(STREAM s)
{
    if (s->inF == NULL) return;
    
    while (fgets(s->buffer, sizeof(s->buffer), s->inF)) {
	char *x = strchr(s->buffer,'#');
	if (x) *x=0;	/* trailing comment on a line */
	s->s = SpaceDrop(s->buffer);
	if (*s->s!='\0')
		break;	/* got something to process */
        ++s->line;	/* nothing there, skip this line */
        if (DEBUG_STREAM) {
	    if (x) *x='#';	/* put it back so we can print the line */
            fprintf(stderr," dropping %s\n", s->buffer);
	}
        *s->buffer = '\0'; s->s = s->buffer;
        
    }
    ++s->line;
    if (DEBUG_STREAM)
        fprintf(stderr," keeping %d/%s\n", strlen(s->s), s->s);
    
}


int IsEndOfStream(STREAM s) 
{
    if (*s->s == '\0') RefreshStreamBuffer(s);

    return (*s->s == '\0');
}


int GetNextToken(STREAM s, char * outBuf, int outSize)
{
    int done = 0;
    int quote = 0;
    int backslash = 0;
    int f = 1;
    
    if (!outSize--) return 0;
    
    if (*s->s == '\0' && s->inF != NULL) {
        if (DEBUG_STREAM)
            fprintf(stderr,"GetNextToken: new line\n");
        *s->buffer = '\0';

        RefreshStreamBuffer(s);
        
    }
    if (!f || *s->s == '\0') {
        if (DEBUG_STREAM)
            fprintf(stderr," END of stream\n");

        *outBuf = '\0';
        return 0;
    }

    if (DEBUG_STREAM)
        fprintf(stderr,"GetNextToken: parsing %s\n", s->s);

    while (*(s->s) && outSize && !done) {
        if (backslash || (quote && *(s->s) != '\\' && *(s->s) != '\"')) {
            // This character is quoted literally
            if (DEBUG_STREAM)
                fprintf(stderr," Literal %x/%c\n", *s->s, *s->s);

            *outBuf++ = *s->s;
            outSize--;
            backslash = 0;
        }
        else switch (*s->s) {
        case '\"': 
            if (DEBUG_STREAM)
                fprintf(stderr," Quote\n");

            quote = !quote;
            break;
        case '\\':
            if (DEBUG_STREAM)
                fprintf(stderr," Backslash\n");
            backslash = 1;
            break;
        default:
            if (*s->s > ' ') {
                if (DEBUG_STREAM)
                    fprintf(stderr," def %x/%c\n", *s->s, *s->s);

                *outBuf++ = *s->s;
                --outSize;
            } else done = 1;
            break;
        }
        (s->s)++;
    }
    s->s = SpaceDrop(s->s);
    *outBuf++ = '\0';
    return 1;
}

void StreamError(STREAM s) 
{
    fprintf(stderr,"Error parsing %s, line %d: ",s->name, s->line);
}


