/* X10front.C
 * unixish Front end for X10 interface assembler x10cmd.c
 *
 $Id: x10front.c,v 1.1 2003/02/17 04:48:33 jimmc Exp $
 *
 $Log: x10front.c,v $
 Revision 1.1  2003/02/17 04:48:33  jimmc
 Program to control CM11 X-10 computer module

 *
 *
 */

#include <stdio.h>
#include <string.h>

#include "x10cmd.h"
#include "serial.h"

static char revision[]="$Revision: 1.1 $"; /* RCS Info */

#define BUFFER_SIZE 64
int debug=0; /* debug & gbl_verbose should be globally visible */
char outputFile[BUFFER_SIZE]="";
char aliasName[BUFFER_SIZE]=ALIAS_DEFAULT;

int Suffix(char *name, char *suffix)
{
  char *s;
  if ( (s=strchr(name,'.')) != NULL)
		 *s='\0';
  strcat(name,suffix);
  return 0;
}

char *On(int bool)
{
  return bool?"ON":"off";
}

int main(int argc,char **argv)
{

  int i;
  char cmd[BUFFER_SIZE];
  char *s;
  char filein[BUFFER_SIZE];
  int pipe=0 ;
  int eex=0 ;
  int machineCode=0;

  InstallSerialDrivers(1<<gbl_port);
/*  SetBaud(gbl_port,600); */
  strncpy(filein,argv[0],BUFFER_SIZE);
  if ( (s=strchr(filein,'.')) != NULL)   /* kill trailing .exe (msdos) */
	 *s='\0';
  strcpy(cmd,(s=strrchr(filein,'\\')) == NULL ? filein : ++s);
					/* kill leading path (msdos) */

  if (argc == 1)  /* no entry: give usage */
     fprintf(stdout,"Usage: %s [-] [-v|q] [-1|2] [-sbtn] [-S time] [-B base] [-d|m file]\n"
		   "       [-a file] [-A line] [-i file] [-I line] [x10 file] [x10 file] ...\n",cmd);
  else {
    while ((--argc) && (*(++argv))[0] == '-') {
      eex=0;
      pipe = (*(argv[0]+1) == '\0' ? 1 : pipe);
      for (s = argv[0]+1; eex==0 && *s !='\0'; s++)
      switch (*s) {
	case 'h' :
	  fprintf(stdout,"%s:  X10 CP290 Assembler & Interface.\n"
			 "       (c) 1994 Erik D. Olson\n",cmd);
	  fprintf(stdout,"Options:\n"
	  "  -h        Print this help\n"
	  "  -v|q      Verbose|quiet mode (default %s)\n"
	  "  -1|2      Set serial port (default COM%d)\n"
	  "  -a file   Specify alias file, ('-a-' =none) (default %s)\n"
	  "  -s        Sync interface clock to system clock\n"
	  "  -S time   Set interface clock\n"
	  "  -B base   Set base housecode (erases events)\n"
	  "  -b        Show base housecode & interface time\n"
	  "  -d|m file Disassemble interface contents to file (m=+machine code)\n"
	  "  -A xline  Assemble & download single timer event\n"
	  "  -I xcmd   Execute single immediate command\n"
	  "  -i file   Execute immediate commands from file (-i-=stdin)\n"
	  "Formats:\n"
	  " xline    prodID  --days-  hr:mn   mode      function  unit,unit,...\n"
	  "            21    MTW--SS  21:30   NORMAL    DIM-B     STEREO,A2,LIGHTS\n\n"
	  " xcmd     function unit,unit,unit...\n"
	  " base     letter A-P\n"
	  " time     DAY hr:mn (ie MON 23:32)\n"
	  " mode     NORMAL SECURITY TODAY TOMORROW or CLEAR\n",
	   gbl_verbose?"verbose":"quiet",gbl_port+1,aliasName);

	  break;
	case '1' : if (gbl_verbose) fprintf(stderr,"COM1\n");gbl_port=0; break;
	case '2' : if (gbl_verbose) fprintf(stderr,"COM2\n");gbl_port=1; break;
	case 's' : SystemToClock(); fprintf(stderr,"system clock->x10\n"); break;
	case 'b' : printf("Upload: %s\n",ClockToStr(RequestClock())); break;
	case 't' : fprintf(stderr,"Yeah, right...\n"); break;

	case 'v' :
	  gbl_verbose=1;
	  fprintf(stderr,"Verbose\n");
	  break;
	case 'q' :
	  gbl_verbose=0;
	  break;
	case 'a' :
	  strcpy(aliasName,*(++s)=='\0'? --argc,*(++argv) : s);
	  if (gbl_verbose)
	    fprintf(stderr,"Alias File is %s\n",aliasName);
	  eex=1; break;
	case 'i' :
	  if (!gbl_aliasMax) ReadAlias(aliasName);
	  i=Immediate(*(++s)=='\0'? --argc,s=*(++argv) : s);
	  if (gbl_verbose)
	    fprintf(stderr,"%sImmediate from %s\n",i<1?ERR_MSG:"",s);
	  eex=1; break;
	case 'I' :
	  if (!gbl_aliasMax) ReadAlias(aliasName);
	  i=ImmediateLine(*(++s)=='\0'? --argc,s=*(++argv) : s);
	  if (gbl_verbose)
	    fprintf(stderr,"%sImmediate line %s\n",i<1?ERR_MSG:"",s);
	  eex=1; break;
	case 'A' :
	  eex=1; if (*(++s)=='\0') --argc,s=*(++argv);
	  if (!gbl_aliasMax) ReadAlias(aliasName);
	  i=AssembleLine(s);
	  if (gbl_verbose)
	    fprintf(stderr,"%sAssemble line %s\n",i<1?ERR_MSG:"",s);
	  break;
	case 'S' :
	  i=StrToClock(*(++s)=='\0'? --argc,s=*(++argv) : s);
	  if (gbl_verbose)
	    fprintf(stderr,"%sSetClock to %s\n",i?ERR_MSG:"",s);
	  eex=1; break;
	case 'B' :
	  if (*(++s)=='\0') --argc,s=*(++argv);
	  fprintf(stderr,"Are you sure you want to erase the contents? ");
	  i=getchar();
	  if (i=='y' || i=='Y') {
	    i=DownloadBaseHousecode(*s);
	    if (gbl_verbose) fprintf(stderr,"%sErase/Set housecode to %c\n",
	      i==1?"":ERR_MSG,*s);
	      if (i!=1) exit(1);
	  }
	  eex=1; break;
	case 'm' :
	  machineCode=1;
	case 'd' :
	  eex=1; if (*(++s)=='\0') --argc,s=*(++argv);
	  if (!gbl_aliasMax) ReadAlias(aliasName);
	  i=DisAssemble(s,machineCode);
	  if (gbl_verbose)
	    fprintf(stderr,"%sDisassemble -> %s\n",i?ERR_MSG:"",s);
	  break;
	default:
	  fprintf(stderr,"Illegal option.  Try %s -h for help.\n",cmd);
	  break;
      } /* switch */
    } /* args with - */
    if (pipe)
    {
      if (!gbl_aliasMax) ReadAlias(aliasName);
      i=Assemble("-");
      if (gbl_verbose)
	fprintf(stderr,"%sAssemble stdin\n",i!=0?ERR_MSG:"");
    }
    else while (argc)
    {
      if (!gbl_aliasMax) ReadAlias(aliasName);
      i=Assemble(*argv);
      if (gbl_verbose)
	fprintf(stderr,"%sAssemble file %s\n",i!=0?ERR_MSG:"",*argv);
      --argc; ++argv;
    } /* file loop */
  } /* usage OK */
  UninstallSerialDrivers();

  return 0;
} /* main */

